package dev.shadowsoffire.apotheosis.advancements;

package .tmp;

import com.google.gson.JsonObject;
import com.mojang.brigadier.exceptions.CommandSyntaxException;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketHelper;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemInstance;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import net.minecraft.advancements.critereon.AbstractCriterionTriggerInstance;
import net.minecraft.advancements.critereon.ContextAwarePredicate;
import net.minecraft.advancements.critereon.DeserializationContext;
import net.minecraft.advancements.critereon.InventoryChangeTrigger;
import net.minecraft.advancements.critereon.ItemPredicate;
import net.minecraft.advancements.critereon.MinMaxBounds;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.TagParser;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.GsonHelper;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraftforge.registries.ForgeRegistries;

public class ExtendedInvTrigger extends InventoryChangeTrigger {
  public InventoryChangeTrigger.TriggerInstance m_7214_(JsonObject json, ContextAwarePredicate andPred, DeserializationContext conditionsParser) {
    JsonObject slots = GsonHelper.m_13841_(json, "slots", new JsonObject());
    MinMaxBounds.Ints occupied = MinMaxBounds.Ints.m_55373_(slots.get("occupied"));
    MinMaxBounds.Ints full = MinMaxBounds.Ints.m_55373_(slots.get("full"));
    MinMaxBounds.Ints empty = MinMaxBounds.Ints.m_55373_(slots.get("empty"));
    ItemPredicate[] predicate = ItemPredicate.m_45055_(json.get("items"));
    if (json.has("apoth"))
      predicate = deserializeApoth(json.getAsJsonObject("apoth")); 
    return new InventoryChangeTrigger.TriggerInstance(andPred, occupied, full, empty, predicate);
  }
  
  ItemPredicate[] deserializeApoth(JsonObject json) {
    String type = json.get("type").getAsString();
    if ("spawn_egg".equals(type))
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> s.m_41720_() instanceof net.minecraft.world.item.SpawnEggItem) }; 
    if ("enchanted".equals(type)) {
      Enchantment ench = json.has("enchantment") ? (Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(json.get("enchantment").getAsString())) : null;
      MinMaxBounds.Ints bound = MinMaxBounds.Ints.m_55373_(json.get("level"));
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> {
              Map<Enchantment, Integer> enchMap = EnchantmentHelper.m_44831_(s);
              if (ench != null)
                return bound.m_55390_(((Integer)enchMap.getOrDefault(ench, Integer.valueOf(0))).intValue()); 
              Objects.requireNonNull(bound);
              return enchMap.values().stream().anyMatch(bound::m_55390_);
            }) };
    } 
    if ("affix".equals(type))
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> !AffixHelper.getAffixes(s).isEmpty()) }; 
    if ("rarity".equals(type)) {
      DynamicHolder<LootRarity> rarity = RarityRegistry.byLegacyId(json.get("rarity").getAsString().toLowerCase(Locale.ROOT));
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> (!AffixHelper.getAffixes(s).isEmpty() && rarity.isBound() && AffixHelper.getRarity(s) == rarity)) };
    } 
    if ("gem_rarity".equals(type)) {
      DynamicHolder<LootRarity> rarity = RarityRegistry.byLegacyId(json.get("rarity").getAsString().toLowerCase(Locale.ROOT));
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> (s.m_41720_() == Adventure.Items.GEM.get() && rarity.isBound() && AffixHelper.getRarity(s) == rarity)) };
    } 
    if ("socket".equals(type))
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> SocketHelper.getGems(s).stream().anyMatch(GemInstance::isValid)) }; 
    if ("nbt".equals(type)) {
      CompoundTag tag;
      try {
        tag = TagParser.m_129359_(GsonHelper.m_13805_(json.get("nbt"), "nbt"));
      } catch (CommandSyntaxException e) {
        throw new RuntimeException(e);
      } 
      return new ItemPredicate[] { (ItemPredicate)new TrueItemPredicate(s -> {
              if (!s.m_41782_())
                return false; 
              for (String key : tag.m_128431_()) {
                if (!tag.m_128423_(key).equals(s.m_41783_().m_128423_(key)))
                  return false; 
              } 
              return true;
            }) };
    } 
    return new ItemPredicate[0];
  }
}
